from bpy.props import EnumProperty, FloatProperty
from bpy.types import Armature, Operator, Context, Event, Mesh, Object, Modifier
from typing import cast, Optional

def bone_items_cb(self, context: Context):
    armature_object = context.object
    armature_data: Armature = cast(Armature, armature_object.data)
    return [(bone.name, bone.name, '') for bone in sorted(armature_data.bones, key=lambda bone: bone.name)]


def find_armature_modifier(mesh_object: Object, armature_object: Object) -> Optional[Modifier]:
    for modifier in mesh_object.modifiers:
        if modifier.type == 'ARMATURE' and modifier.object == armature_object:
            return modifier
    return None


class SMTB_OT_skin_selected_meshes_to_bone(Operator):
    bl_idname = 'smtb.skin_selected_meshes_to_bone'
    bl_label = 'Skin Selected Meshes to Bone'
    bl_description = 'Skin the selected meshes to a bone on the active armature'
    bl_options = {'REGISTER', 'UNDO'}

    # TODO: use enum instead of string.
    bone: EnumProperty(name='Bone', items=bone_items_cb)
    weight: FloatProperty(name='Weight', default=1.0, min=0.0, max=1.0)

    @classmethod
    def poll(cls, context):
        # Return true if the active object is a mesh.
        if context.object is None:
            cls.poll_message_set('No active object')
            return False
        if context.object.type != 'ARMATURE':
            cls.poll_message_set('Active object is not an armature')
            return False
        return True

    def invoke(self, context: Context, event: Event):
        return context.window_manager.invoke_props_dialog(self)

    def draw(self, context: 'Context'):
        layout = self.layout
        flow = layout.grid_flow()
        flow.use_property_split = True
        flow.use_property_decorate = False
        flow.prop(self, 'bone')
        flow.prop(self, 'weight')

    def execute(self, context):
        # For all selected objects:
        armature_object = context.object
        for bpy_object in context.selected_objects:
            if bpy_object.type != 'MESH':
                continue
            # Create a vertex group if it doesn't exist.
            vertex_group = bpy_object.vertex_groups.get(self.bone, None)
            if vertex_group is None:
                vertex_group = bpy_object.vertex_groups.new(name=self.bone)
            # Add all vertices to the vertex group.
            mesh_data = cast(Mesh, bpy_object.data)
            vertex_group.add(range(len(mesh_data.vertices)), self.weight, 'REPLACE')

            # Add an armature modifier if it doesn't exist.
            armature_modifier = find_armature_modifier(bpy_object, armature_object)
            if armature_modifier is None:
                armature_modifier = bpy_object.modifiers.new(name='Armature', type='ARMATURE')
                armature_modifier.object = armature_object
        return {'FINISHED'}


classes = (
    SMTB_OT_skin_selected_meshes_to_bone,
)
